@@ -135,7 +135,7 @@ def read_xyz(row_list, sep=" "):
               "atoms": [],
               "xyz": []}
     if n != len(row_list) - 2:
-        raise Exception("Invalid data.")
+        raise ValueError("Invalid data.")
     for i in range(n):
         sep_row = row_list[i + 2].split(sep)
         row = [item for item in sep_row if item.strip() != ""]
@@ -144,7 +144,7 @@ def read_xyz(row_list, sep=" "):
         x_dict["xyz"].append(coordinates)
     x_dict["xyz"] = np.transpose(x_dict["xyz"])
     if len(x_dict["xyz"]) != 3:
-        raise Exception("Atoms should have 3 coordinates.")
+        raise ValueError("Atoms should have 3 coordinates.")
     return x_dict
 
 
@@ -274,12 +274,12 @@ def write_xyz(x_dict,
     n = x_dict["n"]
     atoms = x_dict["atoms"]
     if n != len(atoms):
-        raise Exception("Invalid number of atoms.")
+        raise ValueError("Invalid number of atoms.")
     if len(x_dict["xyz"]) != 3:
-        raise Exception("Atoms should have 3 coordinates.")
+        raise ValueError("Atoms should have 3 coordinates.")
     for row in x_dict["xyz"]:
         if n != len(row):
-            raise Exception("x_dict['xyz'] is invalid.")
+            raise ValueError("x_dict['xyz'] is invalid.")
     xt_array = np.transpose(x_dict["xyz"])
     if ordering is not None:
         temp_list = [[atoms[i], xt_array[i]] for i in range(n)]
@@ -298,7 +298,7 @@ def write_xyz(x_dict,
         for j in range(3):
             spaces = width - len(xyz[j].split(".")[0])
             if spaces < 0:
-                raise Exception("Too low value for width.")
+                raise ValueError("Too low value for width.")
             xyz[j] = " "*spaces + xyz[j]
         block = (block
                  + "\n"
@@ -520,38 +520,38 @@ def write_zmat(z_dict,
     """
     n = z_dict["n"]
     if n == 1:
-        raise Exception("Monatomic molecule.")
+        raise ValueError("Monatomic molecule.")
     atoms = z_dict["atoms"]
     if n != len(atoms):
-        raise Exception("Invalid number of atoms.")
+        raise ValueError("Invalid number of atoms.")
     if indexing_from not in [0, 1]:
-        raise Exception("indexing_from should be 0 or 1.")
+        raise ValueError("indexing_from should be 0 or 1.")
     ind = [[j + indexing_from for j in item]
            for item in z_dict["indices"]]
     distances = z_dict["distances"]
     if n - 1 != len(distances):
-        raise Exception("Invalid number of distances.")
+        raise ValueError("Invalid number of distances.")
     if len(ind[0]) != 1:
-        raise Exception("z_dict['indices'] is invalid.")
+        raise ValueError("z_dict['indices'] is invalid.")
     if len(ind[1]) != 2:
-        raise Exception("z_dict['indices'] is invalid.")
+        raise ValueError("z_dict['indices'] is invalid.")
     if n > 2:
         angles = z_dict["angles"]
         if n - 2 != len(angles):
-            raise Exception("Invalid number of angles.")
+            raise ValueError("Invalid number of angles.")
         if len(ind[2]) != 3:
-            raise Exception("z_dict['indices'] is invalid.")
+            raise ValueError("z_dict['indices'] is invalid.")
     if n > 3:
         dihedral = z_dict["dihedral"]
         if n - 3 != len(dihedral):
-            raise Exception("Invalid number of dihedral angles.")
+            raise ValueError("Invalid number of dihedral angles.")
         for i in range(3, n):
             if len(ind[i]) != 4:
-                raise Exception("z_dict['indices'] is invalid.")
+                raise ValueError("z_dict['indices'] is invalid.")
     if len(w_vector) != 4:
-        raise Exception("w_vector should contain 4 integers.")
+        raise ValueError("w_vector should contain 4 integers.")
     if c < 0.0:
-        raise Exception("c should be positive.")
+        raise ValueError("c should be positive.")
     # Lists containing the corresponding distance, angle and dihedral
     # angle variable names.
     r_list = []
@@ -607,36 +607,36 @@ def write_zmat(z_dict,
     for i in range(1, n):
         w_i = w_vector[0] + w_atoms - len(str(ind[i][1])) - len(atoms[i])
         if w_i < 0:
-            raise Exception("w_vector["
-                            + str(0)
-                            + "] is too low.")
+            raise ValueError("w_vector["
+                             + str(0)
+                             + "] is too low.")
         w_r = w_vector[1] - len(r_list[i - 1])
         if w_r < 0:
-            raise Exception("w_vector["
-                            + str(1)
-                            + "] is too low.")
+            raise ValueError("w_vector["
+                             + str(1)
+                             + "] is too low.")
         if i > 1 and n > 2:
             w_j = w_vector[0] - len(str(ind[i][2]))
             if w_j < 0:
-                raise Exception("w_vector["
-                                + str(0)
-                                + "] is too low.")
+                raise ValueError("w_vector["
+                                 + str(0)
+                                 + "] is too low.")
             w_a = w_vector[2] - len(a_list[i - 2])
             if w_a < 0:
-                raise Exception("w_vector["
-                                + str(2)
-                                + "] is too low.")
+                raise ValueError("w_vector["
+                                 + str(2)
+                                 + "] is too low.")
         if i > 2 and n > 3:
             w_k = w_vector[0] - len(str(ind[i][3]))
             if w_k < 0:
-                raise Exception("w_vector["
-                                + str(0)
-                                + "] is too low.")
+                raise ValueError("w_vector["
+                                 + str(0)
+                                 + "] is too low.")
             w_d = w_vector[3] - len(d_list[i - 3])
             if w_d < 0:
-                raise Exception("w_vector["
-                                + str(3)
-                                + "] is too low.")
+                raise ValueError("w_vector["
+                                 + str(3)
+                                 + "] is too low.")
         if i == 1:
             geometry = (geometry
                         + "\n"
@@ -686,7 +686,7 @@ def write_zmat(z_dict,
         r = format(distances[i]*c, p)
         w = width - len(r.split(".")[0])
         if w < 0:
-            raise Exception("width is too low.")
+            raise ValueError("width is too low.")
         variables = (variables
                      + r_list[i]
                      + " "*(w_m - len(r_list[i]))
@@ -699,7 +699,7 @@ def write_zmat(z_dict,
             a = format(angles[i], p)
             w = width - len(a.split(".")[0])
             if w < 0:
-                raise Exception("width is too low.")
+                raise ValueError("width is too low.")
             variables = (variables
                          + a_list[i]
                          + " "*(w_m - len(a_list[i]))
@@ -712,7 +712,7 @@ def write_zmat(z_dict,
             d = format(dihedral[i], p)
             w = width - len(d.split(".")[0])
             if w < 0:
-                raise Exception("width is too low.")
+                raise ValueError("width is too low.")
             variables = (variables
                          + d_list[i]
                          + " "*(w_m - len(d_list[i]))
@@ -729,7 +729,7 @@ def write_zmat(z_dict,
     return string
 
 
-def xyz_from_zmat(z_dict):
+def xyz_from_zmat(z_dict, z=179.5):
     r"""Converting the internal coordinate representation of a
     molecule to Cartesian coordinates.
 
@@ -758,6 +758,11 @@ def xyz_from_zmat(z_dict):
         z_dict['dihedral'] : array_like
             Dihedral angles. An optional empty array if the number of
             atoms is less than four.
+    z : int or float, optional
+        The maximum acceptable value of the items in z_dict['angles'].
+        If the angle between three consecutive atoms is bigger than
+        `z`, the atoms are considered collinear. The value of `z`
+        should be bigger than 170 and smaller than 180.
 
     Returns
     -------
@@ -924,41 +929,58 @@ def xyz_from_zmat(z_dict):
     """
     # The number of atoms.
     n = z_dict["n"]
+    if n == 1:
+        raise ValueError("Monatomic molecule.")
     # Atomic coordinates. Initially an n-by-3 matrix of zeros.
     xyz = np.zeros((n, 3))
     # Array of atomic distances.
     r = z_dict["distances"]
+    if min(r) <= 0.0:
+        raise ValueError("Atomic distances are should be positive.")
+    if not 170.0 < z < 180.0:
+        raise ValueError("z should be between 170 and 180.")
     # A degree to radians conversion factor.
     deg_to_rad = np.pi/180.0
-    # Array of angles in radians.
+    # List of angles in radians.
     t = [i*deg_to_rad for i in z_dict["angles"]]
-    # Array of dihedral angles in radians.
+    # List of dihedral angles in radians.
     f = [i*deg_to_rad for i in z_dict["dihedral"]]
     # Block of indices.
     ind = z_dict["indices"]
-    if n == 1:
-        raise Exception("Monatomic molecule.")
+    if n != len(ind):
+        raise ValueError("Invalid number of rows in z_dict['indices'].")
     if type(z_dict["name"]) != str:
-        raise Exception("z_dict['name'] should be an str.")
+        raise TypeError("z_dict['name'] should be an str.")
     if n != len(z_dict["atoms"]):
-        raise Exception("Invalid number of atoms.")
+        raise ValueError("Invalid number of atoms.")
+    for atom in z_dict["atoms"]:
+        if type(atom) != str:
+            raise TypeError("z_dict['atoms'] should be a list of strings.")
     if n - 1 != len(r):
-        raise Exception("Invalid number of distances.")
-    if len(ind[0]) != 1:
-        raise Exception("z_dict['indices'] is invalid.")
-    if len(ind[1]) != 2:
-        raise Exception("z_dict['indices'] is invalid.")
+        raise ValueError("Invalid number of distances.")
+    if list(ind[0]) != [0]:
+        raise ValueError("z_dict['indices'] is invalid.")
+    if list(ind[1]) != [1, 0]:
+        raise ValueError("z_dict['indices'] is invalid.")
     if n > 2:
         if n - 2 != len(t):
-            raise Exception("Invalid number of angles.")
-        if len(ind[2]) != 3:
-            raise Exception("z_dict['indices'] is invalid.")
+            raise ValueError("Invalid number of angles.")
+        if min(z_dict["angles"]) <= 0.0 or max(z_dict["angles"]) > z:
+            raise ValueError("z_dict['angles'] is invalid.")
     if n > 3:
         if n - 3 != len(f):
-            raise Exception("Invalid number of dihedral angles.")
+            raise ValueError("Invalid number of dihedral angles.")
         for i in range(3, n):
+            if ind[i][0] != i or max(ind[i]) != i:
+                raise ValueError("z_dict['indices'] is invalid.")
+            if min(ind[i]) < 0:
+                raise IndexError("An atomic index should be positive.")
             if len(ind[i]) != 4:
-                raise Exception("z_dict['indices'] is invalid.")
+                raise ValueError("z_dict['indices'] is invalid.")
+            for j in range(4):
+                for k in range(j + 1, 4):
+                    if ind[i][j] == ind[i][k]:
+                        raise ValueError("Repeated atomic index.")
     # Calculating the atomic coordinates of the second atom in case of
     # a diatomic molecule.
     if n == 2:
@@ -967,18 +989,18 @@ def xyz_from_zmat(z_dict):
     # in case of a bigger molecule.
     if n > 2:
         # The first atom is connected to the second and the third.
-        if ind[2] == [2, 0, 1]:
+        if list(ind[2]) == [2, 0, 1]:
             xyz[1][0] = -r[0]
             xyz[2][0] = -r[1]*np.cos(t[0])
             xyz[2][1] = r[1]*np.sin(t[0])
         # The first atom is connected to the second, but not connected
         # to the third.
-        elif ind[2] == [2, 1, 0]:
+        elif list(ind[2]) == [2, 1, 0]:
             xyz[0][0] = -r[0]
             xyz[2][0] = -r[1]*np.cos(t[0])
             xyz[2][1] = r[1]*np.sin(t[0])
         else:
-            raise Exception("z_dict['indices'] is invalid.")
+            raise ValueError("z_dict['indices'] is invalid.")
     # Calculating the Cartesian coordinates of the leftover atoms
     # using the NeRF algorithm.
     if n > 3:
@@ -1060,10 +1082,10 @@ def center(a_list):
     # the x, y and z components of the coordinates.
     a_array = np.array(a_list)
     if len(a_list) != 3:
-        raise Exception("a_list should contain 3 rows.")
+        raise ValueError("a_list should contain 3 rows.")
     for item in a_array:
         if n != len(item):
-            raise Exception("Mismatching dimensions.")
+            raise ValueError("Mismatching dimensions.")
     # List containing average x, y and z values.
     xyz_list = [sum(a_array[i])/n for i in range(3)]
     # Element-wise subtraction of the average x, y and z values from
@@ -1121,15 +1143,15 @@ def rmsd(a_list, b_list):
     # 3-by-n dimensional matrix B.
     b_array = np.array(b_list)
     if len(a_list) != 3:
-        raise Exception("a_list should contain 3 rows.")
+        raise ValueError("a_list should contain 3 rows.")
     if len(b_list) != 3:
-        raise Exception("b_list should contain 3 rows.")
+        raise ValueError("b_list should contain 3 rows.")
     for item in a_array:
         if n != len(item):
-            raise Exception("Mismatching dimensions.")
+            raise ValueError("Mismatching dimensions.")
     for item in b_array:
         if n != len(item):
-            raise Exception("Mismatching dimensions.")
+            raise ValueError("Mismatching dimensions.")
     # The sum of the squared errors of matrix A and B.
     s = sum(sum((a_array - b_array)**2))
     # RMSD, the square root of the average of squared errors.
@@ -1303,7 +1325,7 @@ def smsvd(x_list, y_list, imp_rot=False):
         # R = V*UT.
         r_array = np.matmul(v_array, ut_array)
     else:
-        raise Exception("imp_rot can only be True or False.")
+        raise TypeError("imp_rot can only be True or False.")
     # 3-by-n matrix Z representing X after the transformation.
     # Z = R*X.
     z_array = np.matmul(r_array, x_array)
@@ -1388,11 +1410,11 @@ def rmat(e_list):
            [-0.5       ,  0.61237244,  0.61237244]])
     """
     if len(e_list) != 3:
-        raise Exception("e_list should contain three numbers.")
+        raise ValueError("e_list should contain three numbers.")
     for i in e_list:
         if not -180.0 <= i <= 180.0:
-            raise Exception("Euler angles should be between -180 and 180 "
-                            + "degrees.")
+            raise ValueError("Euler angles should be between -180 and 180 "
+                             + "degrees.")
     # x_rad is the angle of rotation about the x-axis in radians.
     x_rad = e_list[0]*np.pi/180.0
     cx = np.cos(x_rad)
@@ -1483,10 +1505,10 @@ def euler(r_list, pz=10):
     """
     r_array = np.array(r_list)
     if len(r_array) != 3:
-        raise Exception("Expected a 3-by-3 dimensional matrix.")
+        raise ValueError("Expected a 3-by-3 dimensional matrix.")
     for i in range(3):
         if len(r_array[i]) != 3:
-            raise Exception("Expected a 3-by-3 dimensional matrix.")
+            raise ValueError("Expected a 3-by-3 dimensional matrix.")
     # Calculating Euler angles in radians.
     if round(abs(r_array[2][0]), pz) != 1.0:
         y_rad = -np.arcsin(r_array[2][0])
@@ -1518,7 +1540,7 @@ def euler(r_list, pz=10):
     # If the reconstructed matrix differs from the original, the
     # original matrix r_array considered an invalid rotation matrix.
     if round(error, pz) != 0.0:
-        raise Exception("Invalid rotation matrix.")
+        raise ValueError("Invalid rotation matrix.")
     return e_array
 
 
@@ -1548,25 +1570,25 @@ class Geom:
     def __init__(self, x_dict):
         self.n = x_dict["n"]
         if type(self.n) != int:
-            raise Exception("x_dict['n'] should be an integer.")
+            raise TypeError("x_dict['n'] should be an integer.")
         self.name = x_dict["name"]
         if type(self.name) != str:
-            raise Exception("x_dict['name'] should be a string.")
+            raise TypeError("x_dict['name'] should be a string.")
         self.atoms = x_dict["atoms"]
         if len(self.atoms) != self.n:
-            raise Exception("x_dict['atoms'] should contain x_dict['n'] "
-                            + "number of atom(s).")
+            raise ValueError("x_dict['atoms'] should contain x_dict['n'] "
+                             + "number of atom(s).")
         for atom in self.atoms:
             if type(atom) != str:
-                raise Exception("x_dict['atoms'] should only contain "
+                raise TypeError("x_dict['atoms'] should only contain "
                                 + "strings.")
         self.xyz = np.array(x_dict["xyz"])
         if len(self.xyz) != 3:
-            raise Exception("x_dict['xyz'] should have 3 rows.")
+            raise ValueError("x_dict['xyz'] should have 3 rows.")
         for row in self.xyz:
             if len(row) != self.n:
-                raise Exception("x_dict['xyz'] should have x_dict['n'] "
-                                + "number of column(s).")
+                raise ValueError("x_dict['xyz'] should have x_dict['n'] "
+                                 + "number of column(s).")
         self.m_dict = {}
         for atom in self.atoms:
             if atom not in self.m_dict:
@@ -1606,7 +1628,7 @@ class Geom:
         [[1, 2, 3, 4], [0], [0], [0], [0]]
         """
         if c < 0.0:
-            raise Exception("c should not be negative.")
+            raise ValueError("c should not be negative.")
         adj_list = []
         for i in range(self.n):
             adj_list.append([])
@@ -1669,7 +1691,7 @@ class Geom:
         1.7473085636486763
         """
         if a == b:
-            raise Exception("a and b should be different.")
+            raise ValueError("a and b should be different.")
         # The n-by-3 dimensional matrix of atomic coordinates.
         xyz = np.transpose(self.xyz)
         # 3 dimensional vectors a and b representing the atomic
@@ -1726,7 +1748,7 @@ class Geom:
         for i in range(3):
             for j in range(i + 1, 3):
                 if abc_list[i] == abc_list[j]:
-                    raise Exception("a, b and c should be different.")
+                    raise ValueError("a, b and c should be different.")
         # The n-by-3 dimensional matrix of atomic coordinates.
         xyz = np.transpose(self.xyz)
         # 3 dimensional vectors a, b and c representing the atomic
@@ -1842,13 +1864,13 @@ class Geom:
         for i in range(4):
             for j in range(i + 1, 4):
                 if abcd_list[i] == abcd_list[j]:
-                    raise Exception("a, b, c and d should be different.")
+                    raise ValueError("a, b, c and d should be different.")
         abc = abs(self.angle(a, b, c))
         bcd = abs(self.angle(b, c, d))
-        if not 170 < z < 180:
-            raise Exception("z should be between 170 and 180.")
+        if not 170.0 < z < 180.0:
+            raise ValueError("z should be between 170 and 180.")
         if abc > z or bcd > z:
-            raise Exception("Three collinear atoms.")
+            raise ValueError("Three collinear atoms.")
         # The n-by-3 dimensional matrix of atomic coordinates.
         xyz = np.transpose(self.xyz)
         # 3 dimensional vectors a, b, c and d representing the atomic
@@ -2063,9 +2085,9 @@ class Geom:
         'ala_4'
         """
         if w <= 1.0:
-            raise Exception("w must be bigger than 1.")
+            raise ValueError("w must be bigger than 1.")
         if z <= 0.0:
-            raise Exception("z must be positive.")
+            raise ValueError("z must be positive.")
         # Number of reference conformers.
         y_len = len(y_list)
         rmsd_list = []
@@ -2081,7 +2103,7 @@ class Geom:
                 rmsd_list.append([y_dict["name"],
                                   rmsd])
             else:
-                raise Exception("Mismatching atoms.")
+                raise ValueError("Mismatching atoms.")
         if y_len > 1:
             # Sorting the elements of the rmsd_list.
             rmsd_list = sorted(rmsd_list, key=lambda x: x[1])
@@ -2354,26 +2376,26 @@ class Geom:
         i_items = []
         for inner_list in i_list:
             if type(inner_list) != list:
-                raise Exception("i_list should contain lists of integers.")
+                raise TypeError("i_list should contain lists of integers.")
             if len(inner_list) < 2:
-                raise Exception("Minimum two indices are required for every "
-                                + "permutation.")
+                raise ValueError("Minimum two indices are required for every "
+                                 + "permutation.")
             i_items = i_items + inner_list
             atom = self.atoms[inner_list[0]]
             for j in range(1, len(inner_list)):
                 if self.atoms[inner_list[j]] != atom:
-                    raise Exception("The permutation of different atoms is "
-                                    + "not possible.")
+                    raise ValueError("The permutation of different atoms is "
+                                     + "not possible.")
         i_count = {}
         for i in i_items:
             if i < 0:
-                raise Exception("The atomic index should be positive.")
+                raise IndexError("The atomic index should be positive.")
             if i not in i_count:
                 i_count[i] = 0
             i_count[i] = i_count[i] + 1
         for v in i_count.values():
             if v > 1:
-                raise Exception("Repeated indices in i_list.")
+                raise ValueError("Repeated indices in i_list.")
         # n-by-3 matrix, the transpose of x_dict['xyz'].
         a_array = np.transpose(self.xyz)
         permuted_list = []
@@ -2465,7 +2487,7 @@ class Geom:
         adj = self.adjacency(c=c)
         for item in adj:
             if len(item) == 0:
-                raise Exception("Fragmented molecule.")
+                raise ValueError("Fragmented molecule.")
         # List temp collects the lowest possible atomic indices
         # corresponding intramolecular distance, angle and dihedral
         # angle calculations.
@@ -2480,7 +2502,7 @@ class Geom:
         # the molecular geometry can not be converted to z-matrix.
         for i in range(1, self.n):
             if adj[i][0] >= i:
-                raise Exception("Can not be converted to z-matrix.")
+                raise ValueError("Can not be converted to z-matrix.")
             temp[i].append(adj[i][0])
         # The third element of the i-th inner list of temp is the
         # smallest possible atomic index for calculating bond angle.
@@ -2543,7 +2565,6 @@ if __name__ == "__main__":
     import os
     import doctest
 
-
     os.chdir(os.path.dirname(__file__))
     doctest.testmod()
     print("Doctest of molgemtools.geom...")
